@@ -22,7 +22,6 @@ from oslo_log import log as logging
 from oslo_utils import netutils
 from oslo_utils import strutils
 from oslo_utils import uuidutils
-import six
 from webob import exc
 
 from neutron_lib._i18n import _
@@ -163,14 +162,14 @@ def validate_values(data, valid_values=None, valid_values_display=None):
                        {'data': data, 'valid_values': valid_values_display})
                 LOG.debug(msg)
                 return msg
-        except TypeError:
+        except TypeError as e:
             # This is a programming error
             msg = (_("'data' of type '%(typedata)s' and 'valid_values' "
                      "of type '%(typevalues)s' are not "
                      "compatible for comparison") %
                    {'typedata': type(data),
                     'typevalues': type(valid_values)})
-            raise TypeError(msg)
+            raise TypeError(msg) from e
     else:
         # This is a programming error
         msg = (_("'valid_values' does not support membership operations"))
@@ -228,7 +227,7 @@ def validate_string(data, max_len=None):
         the given max_len. Otherwise a human readable message indicating why
         the data is invalid.
     """
-    if not isinstance(data, six.string_types):
+    if not isinstance(data, str):
         msg = _("'%s' is not a valid string") % data
         LOG.debug(msg)
         return msg
@@ -378,7 +377,7 @@ def validate_mac_address(data, valid_values=None):
         valid_mac = False
 
     if valid_mac:
-        valid_mac = (not netaddr.EUI(data) in
+        valid_mac = (netaddr.EUI(data) not in
                      map(netaddr.EUI, constants.INVALID_MAC_ADDRESSES))
     # TODO(arosen): The code in this file should be refactored
     # so it catches the correct exceptions. validate_no_whitespace
@@ -869,8 +868,8 @@ def _extract_validator(key_validator):
                 return (validator_name,
                         validators[validator_name],
                         validator_params)
-            except KeyError:
-                raise UndefinedValidator(validator_name)
+            except KeyError as e:
+                raise UndefinedValidator(validator_name) from e
     return None, None, None
 
 
@@ -1117,10 +1116,10 @@ def validate_subnet_service_types(service_types, valid_values=None):
         prefixes += constants.DEVICE_OWNER_COMPUTE_PREFIX
 
         for service_type in service_types:
-            if not isinstance(service_type, six.text_type):
+            if not isinstance(service_type, str):
                 raise n_exc.InvalidInputSubnetServiceType(
                     service_type=service_type)
-            elif not service_type.startswith(tuple(prefixes)):
+            if not service_type.startswith(tuple(prefixes)):
                 raise n_exc.InvalidSubnetServiceType(service_type=service_type)
 
 
@@ -1142,17 +1141,16 @@ def validate_ethertype(ethertype, valid_values=None):
             if ethertype_str == "IPV4":
                 ethertype_str = "IP"
             ethertype_name = 'ETH_TYPE_' + ethertype_str
-            if ethertype_name in os_ken_ethertypes:
-                ethertype = os_ken_ethertypes[ethertype_name]
+            ethertype = os_ken_ethertypes.get(ethertype_name, ethertype)
         except TypeError:
             # Value of ethertype cannot be coerced into a string, like None
             pass
         try:
-            if isinstance(ethertype, six.string_types):
+            if isinstance(ethertype, str):
                 ethertype = int(ethertype, 16)
-            if (isinstance(ethertype, six.integer_types) and
-                    constants.ETHERTYPE_MIN <= ethertype and
-                    ethertype <= constants.ETHERTYPE_MAX):
+            if (isinstance(ethertype, int) and
+                    constants.ETHERTYPE_MIN <= ethertype <=
+                    constants.ETHERTYPE_MAX):
                 return None
         except ValueError:
             pass
