@@ -184,21 +184,6 @@ def retry_db_errors(f):
     return wrapped
 
 
-@contextlib.contextmanager
-def autonested_transaction(sess):
-    """This is a convenience context to not bother with 'nested' parameter.
-
-    :param sess: The database session.
-    :returns: Yields the context transaction from sess.
-    """
-    if sess.is_active:
-        session_context = sess.begin(nested=True)
-    else:
-        session_context = sess.begin(subtransactions=True)
-    with session_context as tx:
-        yield tx
-
-
 def retry_if_session_inactive(context_var_name='context'):
     """Retries only if the session in the context is inactive.
 
@@ -216,18 +201,17 @@ def retry_if_session_inactive(context_var_name='context'):
             # deals with the horrors of finding args of already decorated
             # functions
             ctx_arg_index = p_util.getargspec(f).args.index(context_var_name)
-        except ValueError:
+        except ValueError as e:
             msg = _("Could not find position of var %s") % context_var_name
-            raise RuntimeError(msg)
+            raise RuntimeError(msg) from e
         f_with_retry = retry_db_errors(f)
 
         @functools.wraps(f)
         def wrapped(*args, **kwargs):
             # only use retry wrapper if we aren't nested in an active
             # transaction
-            if context_var_name in kwargs:
-                context = kwargs[context_var_name]
-            else:
+            context = kwargs.get(context_var_name)
+            if context is None:
                 context = args[ctx_arg_index]
             method = f if context.session.is_active else f_with_retry
             return method(*args, **kwargs)
