@@ -38,7 +38,8 @@ PLACEMENT_API_WITH_MEMBER_OF = 'placement 1.3'
 PLACEMENT_API_WITH_NESTED_RESOURCES = 'placement 1.14'
 PLACEMENT_API_RETURN_PROVIDER_BODY = 'placement 1.20'
 PLACEMENT_API_ERROR_CODE = 'placement 1.23'
-PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_ERROR_CODE
+PLACEMENT_API_CONSUMER_GENERATION = 'placement 1.28'
+PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_CONSUMER_GENERATION
 GENERATION_CONFLICT_RETRIES = 10
 
 
@@ -63,8 +64,7 @@ def _check_placement_api_available(f):
                 # to avoid losing information.
                 raise n_exc.PlacementClientError(
                     msg=exc.response.text.replace('\n', ' '))
-            else:
-                raise
+            raise
     return wrapper
 
 
@@ -78,7 +78,7 @@ class UUIDEncoder(jsonutils.JSONEncoder):
     def default(self, o):
         if isinstance(o, uuid.UUID):
             return str(o)
-        return super(UUIDEncoder, self).default(o)
+        return super().default(o)
 
 
 class NoAuthClient(object):
@@ -232,8 +232,7 @@ class PlacementAPIClient(object):
                 if e.response.json()[
                         'errors'][0]['code'] == 'placement.concurrent_update':
                     continue
-                else:
-                    raise
+                raise
 
         raise n_exc.PlacementResourceProviderGenerationConflict(
             resource_provider=resource_provider_uuid,
@@ -276,6 +275,7 @@ class PlacementAPIClient(object):
                                                         the same name.
         :returns: The updated resource provider.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s' % resource_provider['uuid']
         # update does not tolerate if the uuid is repeated in the body
         update_body = resource_provider.copy()
@@ -324,6 +324,7 @@ class PlacementAPIClient(object):
         :raises PlacementResourceProviderNotFound: For failure to find resource
         :returns: The Resource Provider matching the UUID.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s' % resource_provider_uuid
         try:
             return self._get(url).json()
@@ -406,6 +407,7 @@ class PlacementAPIClient(object):
                                                              server side.
         :returns: The updated set of inventory records.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/inventories' % resource_provider_uuid
         body = {
             'resource_provider_generation': resource_provider_generation,
@@ -437,8 +439,7 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def delete_resource_provider_inventory(self, resource_provider_uuid,
@@ -460,12 +461,11 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            elif "No inventory of class" in e.details:
+            if "No inventory of class" in e.details:
                 raise n_exc.PlacementInventoryNotFound(
                     resource_provider=resource_provider_uuid,
                     resource_class=resource_class)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def get_inventory(self, resource_provider_uuid, resource_class):
@@ -488,12 +488,11 @@ class PlacementAPIClient(object):
             if "No resource provider with uuid" in e.details:
                 raise n_exc.PlacementResourceProviderNotFound(
                     resource_provider=resource_provider_uuid)
-            elif _("No inventory of class") in e.details:
+            if _("No inventory of class") in e.details:
                 raise n_exc.PlacementInventoryNotFound(
                     resource_provider=resource_provider_uuid,
                     resource_class=resource_class)
-            else:
-                raise
+            raise
 
     @_check_placement_api_available
     def update_resource_provider_inventory(
@@ -548,6 +547,7 @@ class PlacementAPIClient(object):
         :returns: The list of aggregates together with the resource provider
                   generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/aggregates' % resource_provider_uuid
         try:
             return self._get(url).json()
@@ -569,6 +569,7 @@ class PlacementAPIClient(object):
         :raises PlacementTraitNotFound: If the trait name not found.
         :returns: Evaluates to True if the trait exists.
         """
+        # pylint: disable=raise-missing-from
         url = '/traits/%s' % name
         try:
             return self._get(url)
@@ -593,6 +594,7 @@ class PlacementAPIClient(object):
         :raises PlacementTraitNotFound: If the trait did not exist.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/traits/%s' % (name)
         try:
             self._delete(url)
@@ -625,6 +627,7 @@ class PlacementAPIClient(object):
         :returns: The new traits of the resource provider together with the
                   resource provider generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         body = {
             'resource_provider_generation': resource_provider_generation,
@@ -652,6 +655,7 @@ class PlacementAPIClient(object):
         :returns: The associated traits of the resource provider together
                   with the resource provider generation.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
             return self._get(url).json()
@@ -669,6 +673,7 @@ class PlacementAPIClient(object):
                                                    is not found.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
             self._delete(url)
@@ -691,6 +696,7 @@ class PlacementAPIClient(object):
                                                 is not found.
         :returns: The name of resource class and its set of links.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_classes/%s' % (name)
         try:
             return self._get(url).json()
@@ -727,8 +733,74 @@ class PlacementAPIClient(object):
                                                 is not found.
         :returns: None.
         """
+        # pylint: disable=raise-missing-from
         url = '/resource_classes/%s' % (name)
         try:
             self._delete(url)
         except ks_exc.NotFound:
             raise n_exc.PlacementResourceClassNotFound(resource_class=name)
+
+    @_check_placement_api_available
+    def list_allocations(self, consumer_uuid):
+        """List allocations for the consumer
+
+        :param consumer_uuid: The uuid of the consumer, in case of bound port
+                              owned by a VM, the VM uuid.
+        :returns: All allocation records for the consumer.
+        """
+        url = '/allocations/%s' % consumer_uuid
+        return self._get(url).json()
+
+    def update_qos_minbw_allocation(self, consumer_uuid, minbw_alloc_diff,
+                                    rp_uuid):
+        """Update allocation for QoS minimum bandwidth consumer
+
+        :param consumer_uuid: The uuid of the consumer, in case of bound port
+                              owned by a VM, the VM uuid.
+        :param minbw_alloc_diff: A dict which contains the fields to update
+                            for the allocation under the given resource
+                            provider.
+        :param rp_uuid: uuid of the resource provider for which the
+                        allocations are to be updated.
+        """
+        for i in range(GENERATION_CONFLICT_RETRIES):
+            body = self.list_allocations(consumer_uuid)
+            if not body['allocations']:
+                raise n_exc.PlacementAllocationRemoved(consumer=consumer_uuid)
+            if rp_uuid not in body['allocations']:
+                raise n_exc.PlacementAllocationRpNotExists(
+                    resource_provider=rp_uuid, consumer=consumer_uuid)
+            # Count new min_kbps values based on the diff in alloc_diff
+            for drctn, min_kbps_diff in minbw_alloc_diff.items():
+                orig_kbps = body['allocations'][rp_uuid]['resources'][drctn]
+                new_kbps = orig_kbps + min_kbps_diff
+                if new_kbps > 0:
+                    body['allocations'][rp_uuid]['resources'][drctn] = new_kbps
+                else:
+                    # Remove the resource class if the new value for min_kbps
+                    # is 0
+                    resources = body['allocations'][rp_uuid]['resources']
+                    if len(resources) > 1:
+                        resources.pop(drctn, None)
+                    else:
+                        body['allocations'].pop(rp_uuid)
+            try:
+                # Update allocations has no return body, but leave the loop
+                return self.update_allocation(consumer_uuid, body)
+            except ks_exc.Conflict as e:
+                resp = e.response.json()
+                if resp['errors'][0]['code'] == 'placement.concurrent_update':
+                    continue
+                raise
+        raise n_exc.PlacementAllocationGenerationConflict(
+            consumer=consumer_uuid)
+
+    def update_allocation(self, consumer_uuid, allocations):
+        """Update allocation record for given consumer and rp
+
+        :param consumer_uuid: The uuid of the consumer
+        :param allocations: Dict in the form described in placement API ref:
+                            https://tinyurl.com/yxeuzn6l
+        """
+        url = '/allocations/%s' % consumer_uuid
+        self._put(url, allocations)
