@@ -16,7 +16,6 @@ import uuid
 
 import os_traits
 from oslo_log import log as logging
-import six
 
 from neutron_lib._i18n import _
 from neutron_lib import constants as const
@@ -73,8 +72,6 @@ def six_uuid5(namespace, name):
     # cPython 3.6:
     # https://github.com/python/cpython/blob
     #        /e9e2fd75ccbc6e9a5221cf3525e39e9d042d843f/Lib/uuid.py#L628
-    if six.PY2:
-        name = name.encode('utf-8')
     return uuid.uuid5(namespace=namespace, name=name)
 
 
@@ -128,11 +125,11 @@ def _parse_bandwidth_value(bw_str):
             bw = int(bw_str)
             if bw < 0:
                 raise ValueError()
-    except ValueError:
+    except ValueError as e:
         raise ValueError(_(
             'Cannot parse resource_provider_bandwidths. '
             'Expected: non-negative integer bandwidth value, got: %s') %
-            bw_str)
+            bw_str) from e
     return bw
 
 
@@ -163,10 +160,10 @@ def parse_rp_bandwidths(bandwidths):
             bandwidth += '::'
         try:
             device, egress_str, ingress_str = bandwidth.split(':')
-        except ValueError:
+        except ValueError as e:
             raise ValueError(_(
                 'Cannot parse resource_provider_bandwidths. '
-                'Expected: DEVICE:EGRESS:INGRESS, got: %s') % bandwidth)
+                'Expected: DEVICE:EGRESS:INGRESS, got: %s') % bandwidth) from e
         if device in rv:
             raise ValueError(_(
                 'Cannot parse resource_provider_bandwidths. '
@@ -219,11 +216,11 @@ def parse_rp_inventory_defaults(inventory_defaults):
                 inventory_defaults['allocation_ratio'])
             if inventory_defaults['allocation_ratio'] < 0:
                 raise ValueError()
-    except ValueError:
+    except ValueError as e:
         raise ValueError(_(
             'Cannot parse inventory_defaults.allocation_ratio. '
             'Expected: non-negative float, got: %s') %
-            inventory_defaults['allocation_ratio'])
+            inventory_defaults['allocation_ratio']) from e
 
     # the others are ints
     for key in ('min_unit', 'max_unit', 'reserved', 'step_size'):
@@ -232,12 +229,12 @@ def parse_rp_inventory_defaults(inventory_defaults):
                 inventory_defaults[key] = int(inventory_defaults[key])
                 if inventory_defaults[key] < 0:
                     raise ValueError()
-        except ValueError:
+        except ValueError as e:
             raise ValueError(_(
                 'Cannot parse inventory_defaults.%(key)s. '
                 'Expected: non-negative int, got: %(got)s') % {
                     'key': key,
                     'got': inventory_defaults[key],
-            })
+            }) from e
 
     return inventory_defaults
